/**
*
* Angular Modal Library
* @author TMJ Web Development Team. 2016
*
* Created a default approach modal using bootstrap modal
* @copyright TMJ Philippines BPO Services Inc.
*/

(function() {
    'use strict';

    angular
        .module('tmjModal', ['ui.bootstrap'])
        .constant('MODAL_INSTANCE_CONFIG', modalInstanceConfig())
        .constant('MODAL_CONFIG', modalConfig())
        .constant('BOOSTRAP_CONFIG', bootstrapConfig())
        .controller('DefaultModalInstanceCtrl', DefaultModalInstanceCtrl)
        .service('TMJFormData', TMJFormData)
        .service('Modal', Modal);

    DefaultModalInstanceCtrl.$inject = [
        '$uibModalInstance',
        'attr',
        '$injector',
        'MODAL_INSTANCE_CONFIG',
        '$http',
        'TMJFormData'
    ];

    TMJFormData.$inject = [];

    Modal.$inject = ['$uibModal', 'BOOSTRAP_CONFIG', 'MODAL_CONFIG'];

    function bootstrapConfig() {
        return {
            RESERVED: [
                'size', 'templateUrl', 'controller', 'appendTo', 'animation', 'backdrop',
                'bindToController', 'controllerAs', 'keyboard', 'openedClass'
            ]
        };
    };

    function modalInstanceConfig() {
        return {
            MSG: 'Successfully saved.',
            ERR: {
                'EMPTY_FDATA': 'Please specify data to be submitted'
            }
        }
    }

    function modalConfig() {
        return {
            DEFAULT: {
                'CONTROLLER': 'DefaultModalInstanceCtrl',
                'CONTROLLERAS': 'dmic'
            },
            ERR_MSG: {
                'MISSING': 'Please check your config something is missing',
                'ELEM_NOT_FOUND': 'element not found.'
            }
        };
    }

    function DefaultModalInstanceCtrl($uibModalInstance, attr, $injector,
        MODAL_INSTANCE_CONFIG, $http, TMJFormData) {

        var vm = this;
        vm.$uibModalInstance = $uibModalInstance;
        vm.save = save;
        vm.saveWithFile = saveWithFile;
        vm.close = close;
        // you can override functions stated above
        Object.assign(vm, attr);


        function save() {
            var formData = getFormData();

            return requestSave(formData);
        }

        function saveWithFile() {
            var formData = getFormData();
            formData = TMJFormData.build(formData);

            var config = {
                transformRequest: angular.identity,
                headers: {'Content-Type': undefined}
            };

            return requestSave(formData, config);
        }

        function requestSave(formData, config) {
            if (!formData) return;

            var request = {
                'false': requestHttp,
                'true': requestResource
            };
            // console.log(formData);
            // console.log(config);
            var defer = request[!!vm.resource](formData, config);

            defer.then(function(result) {
                // console.log(result);
                // if not stated to keep open return the result
                if (!vm.keepOpen && !result.error) {
                    // pass the result to the client
                    $uibModalInstance.close(result);
                }

                showMessage(result.msg, result.status);
                // if keep open specify the data to be returned to the client for them to manipulate
                vm.returnData = result;

                // for instance that from adding you want to change url to update/delete
                vm.url = result.url || vm.url;
                // if for instance need to update the input after the transaction
                vm.input = result.input || vm.input;

                if (!vm.resource) return;

                // for instance that from adding you want to change resource action to update/delete
                vm.resource.action = result.resource_action || vm.resource.action;
                // for instance that from adding you want to change resource function
                vm.resource.func = result.resource_func || vm.resource.func;
                // for instance that from adding you want to change resource service
                vm.resource.service = result.resource_service || vm.resource.service;
            }, function(err) {
                showMessage(err, false);
            });
        }

        function showMessage(msg, status) {
            vm.status = status;
            return vm.message = msg || MODAL_INSTANCE_CONFIG.MSG;
        }

        function requestHttp(formData, config) {
            return $http.post(vm.url, formData, config).then(function(result) {
                result = result.data;
                return result;
            });
        }

        function requestResource(formData, config) {
            //inject service or factory dynamically
            var resource_data = vm.resource;

            var resource = !(resource_data.service) ? formData :
                        $injector.get(resource_data.service);

            // function or attr name
            // if no function name means factory return already a resource
            if (resource_data.func) {
                // use for service
                resource = resource[resource_data.func];
            }

            if (typeof resource[resource_data.action] != 'function') {
                return console.error('specified resource is not a function'); //need change to function
            }
            // to change the config headers of the resource change it on declaration in your factory
            var res = resource[resource_data.action](formData);
            if (resource_data.service)
                res = res.$promise;

            return res;
        }

        function getFormData() {
            var formData = angular.copy(vm.input);
            if (formData)
                return formData;

            showMessage(MODAL_INSTANCE_CONFIG.ERR.EMPTY_FDATA, false);
            return false;
        }

        function close() {
            if (!vm.keepOpen || !vm.returnData) return $uibModalInstance.dismiss('cancel');

            $uibModalInstance.close(vm.returnData);
        }

        // function errorCallback(msg) {
        //     return console.error(msg);
        // }
    }

    function TMJFormData() {
        this.build = build;

        function build(data, formData, otherKey) {
            if (!data)
                return;

            var fd = formData || new FormData();

            for (var key in data)
            {
                // use to prevent prototypes to be included in loop (eg. obj.length)
                if (!data.hasOwnProperty(key))
                    continue;

                // if other key exist means multi array
                var field = (!otherKey) ? key : otherKey + '[' + key + ']';

                // if object and instance is not file dig deeper
                if (typeof data[key] == 'object' && !(data[key] instanceof File)) // array or object
                {
                    //recursion call itself change the data
                    this.build(data[key], fd, field);
                    continue;
                }

                fd.append(field, data[key]);
            }

            return fd;
        }
    }

    function Modal($uibModal, BOOSTRAP_CONFIG, MODAL_CONFIG) {
        this.open = open;
        this.checkConfig = checkConfig;

        /**
         * Open the modal
         */
        function open(config) {
            checkConfig(config);

            // DECLARE THE MODEL INSTANCE
            config.controller = config.controller || MODAL_CONFIG.DEFAULT.CONTROLLER;
            // IF NO CONTROLLERAS DEFINED IT WILL BE ALIAS AS DEFAULT AND NEED TO CALL OTHER
            // NON SCOPE FUNCTION LIKE( e.g. default.function_name()/ default.variable)
            config.controllerAs = config.controllerAs || MODAL_CONFIG.DEFAULT.CONTROLLERAS;

            // STATIC VARIABLE HOLDS RESERVE ATTRIBUTE USED BY Bootstrap MODAL
            var staticVar = BOOSTRAP_CONFIG.RESERVED;

            // WILL HOLD OTHER SPECIFIED ATTRIBUTE NOT DECLARED ABOVE
            var resolveAttr = {};

            // ALL ATTR NOT IN STATIC VAR WILL BE PUT IN RESOLVE
            for (var key in config)
            {
                // IF NOT IN STATIC VAR
                if (staticVar.indexOf(key) < 0)
                    resolveAttr[key] = config[key];

            }

            // overwrite resolve
            config.resolve = {
                attr: function() {
                    return resolveAttr;
                },
            };

            // OPEN THE MODAL
            var modalInstance = $uibModal.open(config);
            return modalInstance.result;
        }

        function checkConfig(config) {
            if (!config) return console.error(MODAL_CONFIG.ERR_MSG.MISSING);
        }
    }
})();
